/**
 * 
 */
package de.mcs.tools.sps.emulator.emulator;

import java.util.List;

import de.mcs.tools.sps.emulator.EmulatorInput;
import de.mcs.tools.sps.emulator.EmulatorInternals;
import de.mcs.tools.sps.emulator.EmulatorOutput;
import de.mcs.tools.sps.emulator.SPSCommand;
import de.mcs.tools.sps.emulator.SPSCommandData;
import de.mcs.tools.sps.emulator.SPSEmulator;
import de.mcs.tools.sps.emulator.TOutputCallback;
import de.mcs.tools.sps.emulator.exceptions.WrongProgramSizeException;

/**
 * @author w.klaas
 *
 */
public abstract class AbstractEmulator implements SPSEmulator {

  protected byte[] programMemory;
  private boolean debug;
  private boolean active;
  private TOutputCallback outputCallback;
  private EmulatorInput emulatorInput;
  private EmulatorOutput emulatorOutput;
  private EmulatorInternals emulatorInternals;

  @Override
  public abstract List<SPSCommand> getCommands();

  @Override
  public abstract int getProgramSize();

  @Override
  public void loadProgram(byte[] program) throws WrongProgramSizeException {
    if (program.length > getProgramSize()) {
      throw new WrongProgramSizeException(
          String.format("the program is to long. Only %d bytes are permitted", getProgramSize()));
    }
    this.programMemory = program;
  }

  @Override
  public void startProgram(boolean debug) {
    this.debug = debug;
    if (emulatorInput == null) {
      emulatorInput = createEmulatorInput();
    }
    emulatorOutput = createEmulatorOutput();
    emulatorInternals = createEmulatorInternals();
    reset();
    active = true;
    if (outputCallback != null) {
      outputCallback.onOutput(getEmulatorOutput());
    }
    if (!debug) {
      while (active) {
        callNextCommand();
      }
    }
  }

  protected abstract void callNextCommand();

  protected abstract EmulatorInput createEmulatorInput();

  protected abstract EmulatorOutput createEmulatorOutput();

  protected abstract EmulatorInternals createEmulatorInternals();

  @Override
  public SPSCommandData getCommandData(byte command) {
    SPSCommandData commandData = null;
    byte commandByte = (byte) (command & (byte) 0xF0);

    List<SPSCommand> commands = getCommands();
    for (SPSCommand c : commands) {
      if (c.getCommandByte() == commandByte) {
        for (SPSCommandData cd : c.getCommandDatas()) {
          if (cd.getCommandByte() == command) {
            commandData = cd;
          }
        }
      }
    }
    return commandData;
  }

  protected void runCommand(SPSCommandData commandData) {
    EmulatorOutput emulatorOutput = commandData.doWork(getEmulatorInput(), getEmulatorInternals(), getEmulatorOutput());
    if (outputCallback != null) {
      outputCallback.onOutput(emulatorOutput);
    }
  }

  protected abstract void reset();

  @Override
  public void addCallback(TOutputCallback callback) {
    this.outputCallback = callback;
  }

  @Override
  public void nextStep() {
    callNextCommand();
  }

  @Override
  public boolean isActive() {
    return active;
  }

  @Override
  public void stop() {
    active = false;
  }

  @Override
  public void setEmulatorInput(EmulatorInput input) {
    this.emulatorInput = input;
  };

  @Override
  public EmulatorInput getEmulatorInput() {
    return emulatorInput;
  };

  @Override
  public EmulatorInternals getEmulatorInternals() {
    return emulatorInternals;
  };

  @Override
  public EmulatorOutput getEmulatorOutput() {
    return emulatorOutput;
  };

}
